"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: b6024a41ecb4

from .basesdk import BaseSDK
from mistralai.client import errors, models, utils
from mistralai.client._hooks import HookContext
from mistralai.client.types import OptionalNullable, UNSET
from mistralai.client.utils import get_security_from_env
from mistralai.client.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, List, Mapping, Optional, Union


class Judges(BaseSDK):
    def create(
        self,
        *,
        name: str,
        description: str,
        model_name: str,
        output: Union[
            models.CreateJudgeRequestOutput, models.CreateJudgeRequestOutputTypedDict
        ],
        instructions: str,
        tools: List[str],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Judge:
        r"""Create a new judge

        :param name:
        :param description:
        :param model_name:
        :param output:
        :param instructions:
        :param tools:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateJudgeRequest(
            name=name,
            description=description,
            model_name=model_name,
            output=utils.get_pydantic_model(output, models.CreateJudgeRequestOutput),
            instructions=instructions,
            tools=tools,
        )

        req = self._build_request(
            method="POST",
            path="/v1/observability/judges",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.CreateJudgeRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create_judge_v1_observability_judges_post",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "404", "408", "409", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(models.Judge, http_res)
        if utils.match_response(
            http_res, ["400", "404", "408", "409", "422"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ObservabilityErrorData, http_res
            )
            raise errors.ObservabilityError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        name: str,
        description: str,
        model_name: str,
        output: Union[
            models.CreateJudgeRequestOutput, models.CreateJudgeRequestOutputTypedDict
        ],
        instructions: str,
        tools: List[str],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Judge:
        r"""Create a new judge

        :param name:
        :param description:
        :param model_name:
        :param output:
        :param instructions:
        :param tools:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateJudgeRequest(
            name=name,
            description=description,
            model_name=model_name,
            output=utils.get_pydantic_model(output, models.CreateJudgeRequestOutput),
            instructions=instructions,
            tools=tools,
        )

        req = self._build_request_async(
            method="POST",
            path="/v1/observability/judges",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.CreateJudgeRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create_judge_v1_observability_judges_post",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "404", "408", "409", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(models.Judge, http_res)
        if utils.match_response(
            http_res, ["400", "404", "408", "409", "422"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ObservabilityErrorData, http_res
            )
            raise errors.ObservabilityError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    def list(
        self,
        *,
        type_filter: OptionalNullable[List[models.JudgeOutputType]] = UNSET,
        model_filter: OptionalNullable[List[str]] = UNSET,
        page_size: Optional[int] = 50,
        page: Optional[int] = 1,
        q: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ListJudgesResponse:
        r"""Get judges with optional filtering and search

        :param type_filter: Filter by judge output types
        :param model_filter: Filter by model names
        :param page_size:
        :param page:
        :param q:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetJudgesV1ObservabilityJudgesGetRequest(
            type_filter=type_filter,
            model_filter=model_filter,
            page_size=page_size,
            page=page,
            q=q,
        )

        req = self._build_request(
            method="GET",
            path="/v1/observability/judges",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_judges_v1_observability_judges_get",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "404", "408", "409", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.ListJudgesResponse, http_res)
        if utils.match_response(
            http_res, ["400", "404", "408", "409", "422"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ObservabilityErrorData, http_res
            )
            raise errors.ObservabilityError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        type_filter: OptionalNullable[List[models.JudgeOutputType]] = UNSET,
        model_filter: OptionalNullable[List[str]] = UNSET,
        page_size: Optional[int] = 50,
        page: Optional[int] = 1,
        q: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ListJudgesResponse:
        r"""Get judges with optional filtering and search

        :param type_filter: Filter by judge output types
        :param model_filter: Filter by model names
        :param page_size:
        :param page:
        :param q:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetJudgesV1ObservabilityJudgesGetRequest(
            type_filter=type_filter,
            model_filter=model_filter,
            page_size=page_size,
            page=page,
            q=q,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/observability/judges",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_judges_v1_observability_judges_get",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "404", "408", "409", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.ListJudgesResponse, http_res)
        if utils.match_response(
            http_res, ["400", "404", "408", "409", "422"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ObservabilityErrorData, http_res
            )
            raise errors.ObservabilityError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    def fetch(
        self,
        *,
        judge_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Judge:
        r"""Get judge by id

        :param judge_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetJudgeByIDV1ObservabilityJudgesJudgeIDGetRequest(
            judge_id=judge_id,
        )

        req = self._build_request(
            method="GET",
            path="/v1/observability/judges/{judge_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_judge_by_id_v1_observability_judges__judge_id__get",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "404", "408", "409", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.Judge, http_res)
        if utils.match_response(
            http_res, ["400", "404", "408", "409", "422"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ObservabilityErrorData, http_res
            )
            raise errors.ObservabilityError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    async def fetch_async(
        self,
        *,
        judge_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Judge:
        r"""Get judge by id

        :param judge_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetJudgeByIDV1ObservabilityJudgesJudgeIDGetRequest(
            judge_id=judge_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/observability/judges/{judge_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_judge_by_id_v1_observability_judges__judge_id__get",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "404", "408", "409", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.Judge, http_res)
        if utils.match_response(
            http_res, ["400", "404", "408", "409", "422"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ObservabilityErrorData, http_res
            )
            raise errors.ObservabilityError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    def delete(
        self,
        *,
        judge_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ):
        r"""Delete a judge

        :param judge_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.DeleteJudgeV1ObservabilityJudgesJudgeIDDeleteRequest(
            judge_id=judge_id,
        )

        req = self._build_request(
            method="DELETE",
            path="/v1/observability/judges/{judge_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="delete_judge_v1_observability_judges__judge_id__delete",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "404", "408", "409", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*"):
            return
        if utils.match_response(
            http_res, ["400", "404", "408", "409", "422"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ObservabilityErrorData, http_res
            )
            raise errors.ObservabilityError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    async def delete_async(
        self,
        *,
        judge_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ):
        r"""Delete a judge

        :param judge_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.DeleteJudgeV1ObservabilityJudgesJudgeIDDeleteRequest(
            judge_id=judge_id,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/v1/observability/judges/{judge_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="delete_judge_v1_observability_judges__judge_id__delete",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "404", "408", "409", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*"):
            return
        if utils.match_response(
            http_res, ["400", "404", "408", "409", "422"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ObservabilityErrorData, http_res
            )
            raise errors.ObservabilityError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    def update(
        self,
        *,
        judge_id: str,
        name: str,
        description: str,
        model_name: str,
        output: Union[
            models.UpdateJudgeRequestOutput, models.UpdateJudgeRequestOutputTypedDict
        ],
        instructions: str,
        tools: List[str],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ):
        r"""Update a judge

        :param judge_id:
        :param name:
        :param description:
        :param model_name:
        :param output:
        :param instructions:
        :param tools:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UpdateJudgeV1ObservabilityJudgesJudgeIDPutRequest(
            judge_id=judge_id,
            update_judge_request=models.UpdateJudgeRequest(
                name=name,
                description=description,
                model_name=model_name,
                output=utils.get_pydantic_model(
                    output, models.UpdateJudgeRequestOutput
                ),
                instructions=instructions,
                tools=tools,
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/v1/observability/judges/{judge_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.update_judge_request,
                False,
                False,
                "json",
                models.UpdateJudgeRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="update_judge_v1_observability_judges__judge_id__put",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "404", "408", "409", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*"):
            return
        if utils.match_response(
            http_res, ["400", "404", "408", "409", "422"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ObservabilityErrorData, http_res
            )
            raise errors.ObservabilityError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    async def update_async(
        self,
        *,
        judge_id: str,
        name: str,
        description: str,
        model_name: str,
        output: Union[
            models.UpdateJudgeRequestOutput, models.UpdateJudgeRequestOutputTypedDict
        ],
        instructions: str,
        tools: List[str],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ):
        r"""Update a judge

        :param judge_id:
        :param name:
        :param description:
        :param model_name:
        :param output:
        :param instructions:
        :param tools:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UpdateJudgeV1ObservabilityJudgesJudgeIDPutRequest(
            judge_id=judge_id,
            update_judge_request=models.UpdateJudgeRequest(
                name=name,
                description=description,
                model_name=model_name,
                output=utils.get_pydantic_model(
                    output, models.UpdateJudgeRequestOutput
                ),
                instructions=instructions,
                tools=tools,
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/v1/observability/judges/{judge_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.update_judge_request,
                False,
                False,
                "json",
                models.UpdateJudgeRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="update_judge_v1_observability_judges__judge_id__put",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "404", "408", "409", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*"):
            return
        if utils.match_response(
            http_res, ["400", "404", "408", "409", "422"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ObservabilityErrorData, http_res
            )
            raise errors.ObservabilityError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    def judge_conversation(
        self,
        *,
        judge_id: str,
        messages: List[Dict[str, Any]],
        properties: OptionalNullable[Dict[str, Any]] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.JudgeOutput:
        r"""Run a saved judge on a conversation

        :param judge_id:
        :param messages:
        :param properties:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = (
            models.JudgeConversationV1ObservabilityJudgesJudgeIDLiveJudgingPostRequest(
                judge_id=judge_id,
                judge_conversation_request=models.JudgeConversationRequest(
                    messages=messages,
                    properties=properties,
                ),
            )
        )

        req = self._build_request(
            method="POST",
            path="/v1/observability/judges/{judge_id}/live-judging",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.judge_conversation_request,
                False,
                False,
                "json",
                models.JudgeConversationRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="judge_conversation_v1_observability_judges__judge_id__live_judging_post",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "404", "408", "409", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.JudgeOutput, http_res)
        if utils.match_response(
            http_res, ["400", "404", "408", "409", "422"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ObservabilityErrorData, http_res
            )
            raise errors.ObservabilityError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    async def judge_conversation_async(
        self,
        *,
        judge_id: str,
        messages: List[Dict[str, Any]],
        properties: OptionalNullable[Dict[str, Any]] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.JudgeOutput:
        r"""Run a saved judge on a conversation

        :param judge_id:
        :param messages:
        :param properties:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = (
            models.JudgeConversationV1ObservabilityJudgesJudgeIDLiveJudgingPostRequest(
                judge_id=judge_id,
                judge_conversation_request=models.JudgeConversationRequest(
                    messages=messages,
                    properties=properties,
                ),
            )
        )

        req = self._build_request_async(
            method="POST",
            path="/v1/observability/judges/{judge_id}/live-judging",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.judge_conversation_request,
                False,
                False,
                "json",
                models.JudgeConversationRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="judge_conversation_v1_observability_judges__judge_id__live_judging_post",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "404", "408", "409", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.JudgeOutput, http_res)
        if utils.match_response(
            http_res, ["400", "404", "408", "409", "422"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ObservabilityErrorData, http_res
            )
            raise errors.ObservabilityError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 711b769f2c40

from __future__ import annotations
from .inputentries import InputEntries, InputEntriesTypedDict
from typing import List, Union
from typing_extensions import TypeAliasType


ConversationInputsTypedDict = TypeAliasType(
    "ConversationInputsTypedDict", Union[str, List[InputEntriesTypedDict]]
)


ConversationInputs = TypeAliasType("ConversationInputs", Union[str, List[InputEntries]])



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: e606837a626e

from __future__ import annotations
from .judgeclassificationoutput import (
    JudgeClassificationOutput,
    JudgeClassificationOutputTypedDict,
)
from .judgeregressionoutput import JudgeRegressionOutput, JudgeRegressionOutputTypedDict
from mistralai.client.types import BaseModel
from pydantic import Field
from typing import List, Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


CreateJudgeRequestOutputTypedDict = TypeAliasType(
    "CreateJudgeRequestOutputTypedDict",
    Union[JudgeClassificationOutputTypedDict, JudgeRegressionOutputTypedDict],
)


CreateJudgeRequestOutput = Annotated[
    Union[JudgeClassificationOutput, JudgeRegressionOutput], Field(discriminator="type")
]


class CreateJudgeRequestTypedDict(TypedDict):
    name: str
    description: str
    model_name: str
    output: CreateJudgeRequestOutputTypedDict
    instructions: str
    tools: List[str]


class CreateJudgeRequest(BaseModel):
    name: str

    description: str

    model_name: str

    output: CreateJudgeRequestOutput

    instructions: str

    tools: List[str]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 14518c40a13b

from __future__ import annotations
from mistralai.client.types import BaseModel
import pydantic
from pydantic import ConfigDict
from typing import Any, Dict
from typing_extensions import TypedDict


class ExecutionConfigTypedDict(TypedDict):
    r"""Not typed since mcp config can changed / not stable
    we allow all extra fields and this is a dict
    TODO: once mcp is stable, we need to type this
    """

    type: str


class ExecutionConfig(BaseModel):
    r"""Not typed since mcp config can changed / not stable
    we allow all extra fields and this is a dict
    TODO: once mcp is stable, we need to type this
    """

    model_config = ConfigDict(
        populate_by_name=True, arbitrary_types_allowed=True, extra="allow"
    )
    __pydantic_extra__: Dict[str, Any] = pydantic.Field(init=False)

    type: str

    @property
    def additional_properties(self):
        return self.__pydantic_extra__

    @additional_properties.setter
    def additional_properties(self, value):
        self.__pydantic_extra__ = value  # pyright: ignore[reportIncompatibleVariableOverride]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: b5819dd5f981

from __future__ import annotations
from mistralai.client.types import UnrecognizedStr
from typing import Literal, Union


ResourceVisibility = Union[
    Literal[
        "shared_global",
        "shared_org",
        "shared_workspace",
        "private",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from mistralai.azure.client.types import UnrecognizedStr
from typing import Literal, Union


ImageDetail = Union[
    Literal[
        "low",
        "auto",
        "high",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 81651291187a

from __future__ import annotations
from .classifierfinetuningjob import (
    ClassifierFineTuningJob,
    ClassifierFineTuningJobTypedDict,
)
from .completionfinetuningjob import (
    CompletionFineTuningJob,
    CompletionFineTuningJobTypedDict,
)
from .legacyjobmetadata import LegacyJobMetadata, LegacyJobMetadataTypedDict
from functools import partial
from mistralai.client.types import BaseModel
from mistralai.client.utils.unions import parse_open_union
from pydantic import ConfigDict
from pydantic.functional_validators import BeforeValidator
from typing import Any, Literal, Union
from typing_extensions import Annotated, TypeAliasType


ResponseTypedDict = TypeAliasType(
    "ResponseTypedDict",
    Union[ClassifierFineTuningJobTypedDict, CompletionFineTuningJobTypedDict],
)


class UnknownResponse(BaseModel):
    r"""A Response variant the SDK doesn't recognize. Preserves the raw payload."""

    job_type: Literal["UNKNOWN"] = "UNKNOWN"
    raw: Any
    is_unknown: Literal[True] = True

    model_config = ConfigDict(frozen=True)


_RESPONSE_VARIANTS: dict[str, Any] = {
    "classifier": ClassifierFineTuningJob,
    "completion": CompletionFineTuningJob,
}


Response = Annotated[
    Union[ClassifierFineTuningJob, CompletionFineTuningJob, UnknownResponse],
    BeforeValidator(
        partial(
            parse_open_union,
            disc_key="job_type",
            variants=_RESPONSE_VARIANTS,
            unknown_cls=UnknownResponse,
            union_name="Response",
        )
    ),
]


JobsAPIRoutesFineTuningCreateFineTuningJobResponseTypedDict = TypeAliasType(
    "JobsAPIRoutesFineTuningCreateFineTuningJobResponseTypedDict",
    Union[LegacyJobMetadataTypedDict, ResponseTypedDict],
)
r"""OK"""


JobsAPIRoutesFineTuningCreateFineTuningJobResponse = TypeAliasType(
    "JobsAPIRoutesFineTuningCreateFineTuningJobResponse",
    Union[LegacyJobMetadata, Response],
)
r"""OK"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 2164b6526a81

from __future__ import annotations
from .usageinfo_dollar_defs import UsageInfoDollarDefs, UsageInfoDollarDefsTypedDict
from mistralai.client.types import BaseModel
from mistralai.client.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class SpeechStreamDoneTypedDict(TypedDict):
    usage: UsageInfoDollarDefsTypedDict
    type: Literal["speech.audio.done"]


class SpeechStreamDone(BaseModel):
    usage: UsageInfoDollarDefs

    type: Annotated[
        Annotated[
            Literal["speech.audio.done"],
            AfterValidator(validate_const("speech.audio.done")),
        ],
        pydantic.Field(alias="type"),
    ] = "speech.audio.done"


try:
    SpeechStreamDone.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from typing import Literal


BuiltInConnectors = Literal[
    "web_search",
    "web_search_premium",
    "code_interpreter",
    "image_generation",
    "document_library",
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 24d6a0861d4b

from __future__ import annotations
from mistralai.client.types import UnrecognizedStr
from typing import Literal, Union


ConversationSource = Union[
    Literal[
        "EXPLORER",
        "UPLOADED_FILE",
        "DIRECT_INPUT",
        "PLAYGROUND",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 76db69eebe41

from __future__ import annotations
from .builtinconnectors import BuiltInConnectors
from datetime import datetime
from mistralai.client.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from mistralai.client.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Any, Dict, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


ToolExecutionEntryNameTypedDict = TypeAliasType(
    "ToolExecutionEntryNameTypedDict", Union[BuiltInConnectors, str]
)


ToolExecutionEntryName = TypeAliasType(
    "ToolExecutionEntryName", Union[BuiltInConnectors, str]
)


class ToolExecutionEntryTypedDict(TypedDict):
    name: ToolExecutionEntryNameTypedDict
    arguments: str
    object: Literal["entry"]
    type: Literal["tool.execution"]
    created_at: NotRequired[datetime]
    completed_at: NotRequired[Nullable[datetime]]
    agent_id: NotRequired[Nullable[str]]
    model: NotRequired[Nullable[str]]
    id: NotRequired[str]
    info: NotRequired[Dict[str, Any]]


class ToolExecutionEntry(BaseModel):
    name: ToolExecutionEntryName

    arguments: str

    object: Annotated[
        Annotated[Optional[Literal["entry"]], AfterValidator(validate_const("entry"))],
        pydantic.Field(alias="object"),
    ] = "entry"

    type: Annotated[
        Annotated[
            Optional[Literal["tool.execution"]],
            AfterValidator(validate_const("tool.execution")),
        ],
        pydantic.Field(alias="type"),
    ] = "tool.execution"

    created_at: Optional[datetime] = None

    completed_at: OptionalNullable[datetime] = UNSET

    agent_id: OptionalNullable[str] = UNSET

    model: OptionalNullable[str] = UNSET

    id: Optional[str] = None

    info: Optional[Dict[str, Any]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "object",
                "type",
                "created_at",
                "completed_at",
                "agent_id",
                "model",
                "id",
                "info",
            ]
        )
        nullable_fields = set(["completed_at", "agent_id", "model"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    ToolExecutionEntry.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 26c8aadf416a

from __future__ import annotations
from .builtinconnectors import BuiltInConnectors
from mistralai.client.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from mistralai.client.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


ToolFileChunkToolTypedDict = TypeAliasType(
    "ToolFileChunkToolTypedDict", Union[BuiltInConnectors, str]
)


ToolFileChunkTool = TypeAliasType("ToolFileChunkTool", Union[BuiltInConnectors, str])


class ToolFileChunkTypedDict(TypedDict):
    tool: ToolFileChunkToolTypedDict
    file_id: str
    type: Literal["tool_file"]
    file_name: NotRequired[Nullable[str]]
    file_type: NotRequired[Nullable[str]]


class ToolFileChunk(BaseModel):
    tool: ToolFileChunkTool

    file_id: str

    type: Annotated[
        Annotated[
            Optional[Literal["tool_file"]], AfterValidator(validate_const("tool_file"))
        ],
        pydantic.Field(alias="type"),
    ] = "tool_file"

    file_name: OptionalNullable[str] = UNSET

    file_type: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["type", "file_name", "file_type"])
        nullable_fields = set(["file_name", "file_type"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    ToolFileChunk.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: ce5dce4dced2

from __future__ import annotations
from mistralai.client.types import BaseModel
from mistralai.client.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class AudioChunkTypedDict(TypedDict):
    input_audio: str
    type: Literal["input_audio"]


class AudioChunk(BaseModel):
    input_audio: str

    type: Annotated[
        Annotated[
            Literal["input_audio"], AfterValidator(validate_const("input_audio"))
        ],
        pydantic.Field(alias="type"),
    ] = "input_audio"


try:
    AudioChunk.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: cb6e8a484a38

from __future__ import annotations
from .observabilityerrorcode import ObservabilityErrorCode
from mistralai.client.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing_extensions import TypedDict


class ObservabilityErrorDetailTypedDict(TypedDict):
    message: str
    error_code: Nullable[ObservabilityErrorCode]


class ObservabilityErrorDetail(BaseModel):
    message: str

    error_code: Nullable[ObservabilityErrorCode]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: f5918c34f1c7

from __future__ import annotations
from mistralai.client.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from mistralai.client.utils import FieldMetadata, PathParamMetadata, QueryParamMetadata
from pydantic import model_serializer
from typing import Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


AgentsAPIV1AgentsGetAgentVersionTypedDict = TypeAliasType(
    "AgentsAPIV1AgentsGetAgentVersionTypedDict", Union[int, str]
)


AgentsAPIV1AgentsGetAgentVersion = TypeAliasType(
    "AgentsAPIV1AgentsGetAgentVersion", Union[int, str]
)


class AgentsAPIV1AgentsGetRequestTypedDict(TypedDict):
    agent_id: str
    agent_version: NotRequired[Nullable[AgentsAPIV1AgentsGetAgentVersionTypedDict]]


class AgentsAPIV1AgentsGetRequest(BaseModel):
    agent_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    agent_version: Annotated[
        OptionalNullable[AgentsAPIV1AgentsGetAgentVersion],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["agent_version"])
        nullable_fields = set(["agent_version"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""OTEL conventions for gen AI may be found at:

https://opentelemetry.io/docs/specs/semconv/gen-ai/gen-ai-spans/
https://opentelemetry.io/docs/specs/semconv/gen-ai/gen-ai-agent-spans/
"""

import copy
import json
import logging
import os
import traceback
from datetime import datetime, timezone
from enum import Enum
from typing import Any

import httpx
import opentelemetry.semconv._incubating.attributes.gen_ai_attributes as gen_ai_attributes
import opentelemetry.semconv._incubating.attributes.http_attributes as http_attributes
import opentelemetry.semconv.attributes.error_attributes as error_attributes
import opentelemetry.semconv.attributes.server_attributes as server_attributes
from opentelemetry import context as context_api
from opentelemetry import propagate, trace
from opentelemetry.baggage import get_baggage
from opentelemetry.trace import Span, Status, StatusCode, Tracer, set_span_in_context

from .serialization import (
    serialize_input_message,
    serialize_output_message,
    serialize_tool_definition,
)
from .streaming import accumulate_chunks_to_response_dict, parse_sse_chunks

logger = logging.getLogger(__name__)


OTEL_SERVICE_NAME: str = "mistralai_sdk"
MISTRAL_SDK_OTEL_TRACER_NAME: str = OTEL_SERVICE_NAME + "_tracer"

MISTRAL_SDK_DEBUG_TRACING: bool = (
    os.getenv("MISTRAL_SDK_DEBUG_TRACING", "false").lower() == "true"
)
DEBUG_HINT: str = "To see detailed tracing logs, set MISTRAL_SDK_DEBUG_TRACING=true."


class MistralAIAttributes:
    MISTRAL_AI_OCR_USAGE_PAGES_PROCESSED = "mistral_ai.ocr.usage.pages_processed"
    MISTRAL_AI_OCR_USAGE_DOC_SIZE_BYTES = "mistral_ai.ocr.usage.doc_size_bytes"
    MISTRAL_AI_ERROR_CODE = "mistral_ai.error.code"


class MistralAINameValues(Enum):
    OCR = "ocr"


class TracingErrors(Exception, Enum):
    FAILED_TO_CREATE_SPAN_FOR_REQUEST = "Failed to create span for request."
    FAILED_TO_ENRICH_SPAN_WITH_RESPONSE = "Failed to enrich span with response."
    FAILED_TO_HANDLE_ERROR_IN_SPAN = "Failed to handle error in span."
    FAILED_TO_END_SPAN = "Failed to end span."

    def __str__(self):
        return str(self.value)


class GenAISpanEnum(str, Enum):
    CONVERSATION = "conversation"
    VALIDATE_RUN = "validate_run"


def parse_time_to_nanos(ts: str) -> int:
    dt = datetime.fromisoformat(ts.replace("Z", "+00:00")).astimezone(timezone.utc)
    return int(dt.timestamp() * 1e9)


def _infer_gen_ai_operation_name(
    operation_id: str,
) -> gen_ai_attributes.GenAiOperationNameValues | None:
    """Infer the GenAI operation name from the operation_id using rule-based matching."""
    if "chat_completion" in operation_id or operation_id == "stream_chat":
        return gen_ai_attributes.GenAiOperationNameValues.CHAT
    if (
        "agents_create" in operation_id or "agents_update" in operation_id
    ) and "alias" not in operation_id:
        return gen_ai_attributes.GenAiOperationNameValues.CREATE_AGENT
    if "agents_completion" in operation_id or operation_id == "stream_agents":
        return gen_ai_attributes.GenAiOperationNameValues.INVOKE_AGENT
    if "conversations" in operation_id and any(
        action in operation_id for action in ("start", "append", "restart")
    ):
        return gen_ai_attributes.GenAiOperationNameValues.INVOKE_AGENT
    if "fim" in operation_id:
        return gen_ai_attributes.GenAiOperationNameValues.TEXT_COMPLETION
    if "embeddings" in operation_id:
        return gen_ai_attributes.GenAiOperationNameValues.EMBEDDINGS
    if "ocr_post" in operation_id:
        return gen_ai_attributes.GenAiOperationNameValues.GENERATE_CONTENT
    # TODO: Handle transcriptions (audio_api_v1_transcriptions_post[_stream])
    return None


def _build_genai_span_name(
    gen_ai_op: gen_ai_attributes.GenAiOperationNameValues, body: dict[str, Any]
) -> str:
    """Build span name per GenAI semantic conventions.

    - Chat/text_completion/embeddings: "{operation_name} {model}"
    - create_agent/invoke_agent: "{operation_name} {agent_name}"
    - execute_tool: "execute_tool {gen_ai.tool.name}"
    """
    op_name = gen_ai_op.value
    if gen_ai_op in {
        gen_ai_attributes.GenAiOperationNameValues.CREATE_AGENT,
        gen_ai_attributes.GenAiOperationNameValues.INVOKE_AGENT,
    }:
        agent_name = body.get("name", "")
        return f"{op_name} {agent_name}" if agent_name else op_name
    if gen_ai_op is gen_ai_attributes.GenAiOperationNameValues.EXECUTE_TOOL:
        tool_name = body.get("name", "")
        return f"{op_name} {tool_name}" if tool_name else op_name
    model = body.get("model", "")
    return f"{op_name} {model}" if model else op_name


def set_available_attributes(span: Span, attributes: dict[str, Any]) -> None:
    for attribute, value in attributes.items():
        if value:
            span.set_attribute(attribute, value)


def _set_http_attributes(span: Span, operation_id: str, request: httpx.Request) -> None:
    """Set HTTP and server attributes on the span."""
    if not request.url.port:
        # From httpx doc:
        # Note that the URL class performs port normalization as per the WHATWG spec.
        # Default ports for "http", "https", "ws", "wss", and "ftp" schemes are always treated as None.
        # Handling default ports since most of the time we are using https
        if request.url.scheme == "https":
            port = 443
        elif request.url.scheme == "http":
            port = 80
        else:
            port = -1
    else:
        port = request.url.port

    span.set_attributes(
        {
            http_attributes.HTTP_REQUEST_METHOD: request.method,
            http_attributes.HTTP_URL: str(request.url),
            server_attributes.SERVER_ADDRESS: request.headers.get("host", ""),
            server_attributes.SERVER_PORT: port,
        }
    )


def _enrich_request_genai_attrs(
    span: Span,
    gen_ai_op: gen_ai_attributes.GenAiOperationNameValues,
    request_body: dict[str, Any],
) -> None:
    """Set GenAI request attributes: model params, input messages, tool definitions."""
    # Update span name per GenAI semantic conventions, now that we have the parsed request body.
    span.update_name(_build_genai_span_name(gen_ai_op, request_body))

    attributes = {
        gen_ai_attributes.GEN_AI_REQUEST_CHOICE_COUNT: request_body.get("n"),
        gen_ai_attributes.GEN_AI_REQUEST_ENCODING_FORMATS: request_body.get(
            "encoding_formats"
        ),
        gen_ai_attributes.GEN_AI_REQUEST_FREQUENCY_PENALTY: request_body.get(
            "frequency_penalty"
        ),
        gen_ai_attributes.GEN_AI_REQUEST_MAX_TOKENS: request_body.get("max_tokens"),
        gen_ai_attributes.GEN_AI_REQUEST_MODEL: request_body.get("model"),
        gen_ai_attributes.GEN_AI_REQUEST_PRESENCE_PENALTY: request_body.get(
            "presence_penalty"
        ),
        gen_ai_attributes.GEN_AI_REQUEST_SEED: request_body.get("random_seed"),
        gen_ai_attributes.GEN_AI_REQUEST_STOP_SEQUENCES: request_body.get("stop"),
        gen_ai_attributes.GEN_AI_REQUEST_TEMPERATURE: request_body.get("temperature"),
        gen_ai_attributes.GEN_AI_REQUEST_TOP_P: request_body.get("top_p"),
        gen_ai_attributes.GEN_AI_REQUEST_TOP_K: request_body.get("top_k"),
    }

    # Chat/agent completion API uses messages in request body; conversation API uses inputs
    input_messages = request_body.get("messages") or request_body.get("inputs")
    if isinstance(input_messages, str):
        attributes[gen_ai_attributes.GEN_AI_INPUT_MESSAGES] = [
            serialize_input_message({"role": "user", "content": input_messages})
        ]
    elif isinstance(input_messages, list):
        attributes[gen_ai_attributes.GEN_AI_INPUT_MESSAGES] = list(
            map(serialize_input_message, input_messages)
        )
    # Tool definitions
    if tools := request_body.get("tools"):
        attributes[gen_ai_attributes.GEN_AI_TOOL_DEFINITIONS] = list(
            filter(None, map(serialize_tool_definition, tools))
        )
    # TODO: For agent start conversation, add agent id and version attributes here ?

    set_available_attributes(span, attributes)


def enrich_span_from_request(
    span: Span, operation_id: str, request: httpx.Request
) -> Span:
    _set_http_attributes(span, operation_id, request)

    gen_ai_op = _infer_gen_ai_operation_name(operation_id)
    if gen_ai_op is None:
        return span

    span.set_attributes(
        {
            gen_ai_attributes.GEN_AI_OPERATION_NAME: gen_ai_op.value,
            gen_ai_attributes.GEN_AI_PROVIDER_NAME: gen_ai_attributes.GenAiProviderNameValues.MISTRAL_AI.value,
        }
    )

    if request.content:
        request_body = json.loads(request.content)
        _enrich_request_genai_attrs(span, gen_ai_op, request_body)

    return span


def _enrich_response_genai_attrs(
    span: Span,
    gen_ai_op: gen_ai_attributes.GenAiOperationNameValues,
    response_data: dict[str, Any],
) -> None:
    """Set common GenAI response attributes: response ID, model, choices, usage."""
    attributes: dict[str, Any] = {}

    if gen_ai_op is not gen_ai_attributes.GenAiOperationNameValues.CREATE_AGENT:
        # id has another meaning for create agent operation (id of the agent)
        attributes[gen_ai_attributes.GEN_AI_RESPONSE_ID] = response_data.get("id")
    attributes[gen_ai_attributes.GEN_AI_RESPONSE_MODEL] = response_data.get("model")

    # Finish reasons and output messages from choices
    choices = response_data.get("choices", [])
    finish_reasons = [c.get("finish_reason") for c in choices if c.get("finish_reason")]
    if finish_reasons:
        attributes[gen_ai_attributes.GEN_AI_RESPONSE_FINISH_REASONS] = finish_reasons
    if choices:
        attributes[gen_ai_attributes.GEN_AI_OUTPUT_MESSAGES] = list(
            map(serialize_output_message, choices)
        )

    # Usage
    usage = response_data.get("usage", {})
    if usage:
        attributes.update(
            {
                gen_ai_attributes.GEN_AI_USAGE_INPUT_TOKENS: usage.get(
                    "prompt_tokens", 0
                ),
                gen_ai_attributes.GEN_AI_USAGE_OUTPUT_TOKENS: usage.get(
                    "completion_tokens", 0
                ),
            }
        )

    set_available_attributes(span, attributes)


def _enrich_create_agent(span: Span, response_data: dict[str, Any]) -> None:
    """Set agent-specific attributes from create_agent response.

    Semantics: https://opentelemetry.io/docs/specs/semconv/gen-ai/gen-ai-agent-spans/#create-agent-span
    """
    agent_attributes = {
        gen_ai_attributes.GEN_AI_AGENT_DESCRIPTION: response_data.get("description"),
        gen_ai_attributes.GEN_AI_AGENT_ID: response_data.get("id"),
        gen_ai_attributes.GEN_AI_AGENT_NAME: response_data.get("name"),
        # As of 2026-03-02: in convention, but not yet in opentelemetry-semantic-conventions
        "gen_ai.agent.version": str(response_data.get("version")),
        gen_ai_attributes.GEN_AI_REQUEST_MODEL: response_data.get("model"),
        gen_ai_attributes.GEN_AI_SYSTEM_INSTRUCTIONS: response_data.get("instructions"),
    }
    set_available_attributes(span, agent_attributes)


def _create_tool_execution_child_span(
    tracer: trace.Tracer, parent_context: context_api.Context, output: dict[str, Any]
) -> None:
    """Create a child span for a tool.execution conversation output."""
    start_ns = parse_time_to_nanos(output["created_at"])
    end_ns = parse_time_to_nanos(output["completed_at"])
    op_name = gen_ai_attributes.GenAiOperationNameValues.EXECUTE_TOOL
    span_name = _build_genai_span_name(op_name, output)
    child_span = tracer.start_span(
        span_name, start_time=start_ns, context=parent_context
    )
    child_span.set_attributes({"agent.trace.public": ""})
    tool_arguments = output.get("arguments")
    # The tool call result is in the "info" field, if provided
    tool_result = output.get("info")
    tool_attributes = {
        gen_ai_attributes.GEN_AI_OPERATION_NAME: op_name.value,
        gen_ai_attributes.GEN_AI_PROVIDER_NAME: gen_ai_attributes.GenAiProviderNameValues.MISTRAL_AI.value,
        gen_ai_attributes.GEN_AI_TOOL_CALL_ID: output.get("id"),
        gen_ai_attributes.GEN_AI_TOOL_CALL_ARGUMENTS: tool_arguments
        if isinstance(tool_arguments, str)
        else (json.dumps(tool_arguments) if tool_arguments else None),
        gen_ai_attributes.GEN_AI_TOOL_CALL_RESULT: tool_result
        and json.dumps(tool_result),
        gen_ai_attributes.GEN_AI_TOOL_NAME: output.get("name"),
        gen_ai_attributes.GEN_AI_TOOL_TYPE: "extension",
    }
    set_available_attributes(child_span, tool_attributes)
    child_span.end(end_time=end_ns)


def _create_message_output_child_span(
    tracer: trace.Tracer, parent_context: context_api.Context, output: dict[str, Any]
) -> None:
    """Create a child span for a message.output conversation output."""
    start_ns = parse_time_to_nanos(output["created_at"])
    end_ns = parse_time_to_nanos(output["completed_at"])
    op_name = gen_ai_attributes.GenAiOperationNameValues.CHAT
    span_name = _build_genai_span_name(op_name, output)
    child_span = tracer.start_span(
        span_name, start_time=start_ns, context=parent_context
    )
    child_span.set_attributes({"agent.trace.public": ""})
    # Wrap the flat conversation output as a choice dict so we
    # can reuse serialize_output_message (which also handles
    # tool_calls, not just content).
    choice_wrapper: dict = {
        "message": output,
        "finish_reason": output.get("finish_reason", ""),
    }
    message_attributes = {
        gen_ai_attributes.GEN_AI_OPERATION_NAME: op_name.value,
        gen_ai_attributes.GEN_AI_PROVIDER_NAME: gen_ai_attributes.GenAiProviderNameValues.MISTRAL_AI.value,
        gen_ai_attributes.GEN_AI_RESPONSE_ID: output.get("id"),
        gen_ai_attributes.GEN_AI_AGENT_ID: output.get("agent_id"),
        gen_ai_attributes.GEN_AI_RESPONSE_MODEL: output.get("model"),
        gen_ai_attributes.GEN_AI_OUTPUT_MESSAGES: [
            serialize_output_message(choice_wrapper)
        ],
    }
    set_available_attributes(child_span, message_attributes)
    child_span.end(end_time=end_ns)


def _enrich_invoke_agent(
    tracer: trace.Tracer, span: Span, response_data: dict[str, Any]
) -> None:
    """Set invoke_agent attributes and create child spans for conversation outputs."""
    conversation_attributes = {
        gen_ai_attributes.GEN_AI_CONVERSATION_ID: response_data.get("conversation_id"),
        # We don't have more agent attributes available in the response data
        # (agent id, name, version, description). For start conversation operation,
        # we could get it from the request; see associated TODO
    }
    set_available_attributes(span, conversation_attributes)

    outputs = response_data.get("outputs", [])
    parent_context = set_span_in_context(span)
    for output in outputs:
        output_type = output.get("type")
        if not output_type:
            continue  # Safety net
        if output_type == "function.call":
            # handled in the extra.run.tools.create_function_result function
            continue
        elif output_type == "tool.execution":
            _create_tool_execution_child_span(tracer, parent_context, output)
        elif output_type == "message.output":
            _create_message_output_child_span(tracer, parent_context, output)
        # TODO: do type agent.handoff


def _enrich_ocr(span: Span, response_data: dict[str, Any]) -> None:
    """Set OCR-specific usage attributes."""
    usage_info = response_data.get("usage_info", {})
    ocr_attributes = {
        MistralAIAttributes.MISTRAL_AI_OCR_USAGE_PAGES_PROCESSED: usage_info.get(
            "pages_processed"
        ),
        MistralAIAttributes.MISTRAL_AI_OCR_USAGE_DOC_SIZE_BYTES: usage_info.get(
            "doc_size_bytes"
        ),
    }
    set_available_attributes(span, ocr_attributes)


def _enrich_span_from_response(
    tracer: Tracer,
    span: Span,
    operation_id: str,
    response_data: dict[str, Any],
) -> None:
    """Enrich span with GenAI response attributes and operation-specific data.

    Used by both the non-streaming and streaming paths so that the same
    attributes are set regardless of response type.
    """
    gen_ai_op = _infer_gen_ai_operation_name(operation_id)
    if gen_ai_op is None:
        return

    _enrich_response_genai_attrs(span, gen_ai_op, response_data)

    if gen_ai_op is gen_ai_attributes.GenAiOperationNameValues.CREATE_AGENT:
        _enrich_create_agent(span, response_data)
    elif gen_ai_op is gen_ai_attributes.GenAiOperationNameValues.INVOKE_AGENT:
        _enrich_invoke_agent(tracer, span, response_data)

    if operation_id == "ocr_v1_ocr_post":
        _enrich_ocr(span, response_data)


def get_or_create_otel_tracer() -> tuple[bool, Tracer]:
    """
    Get a tracer from the current TracerProvider.

    The SDK does not set up its own TracerProvider - it relies on the application
    to configure OpenTelemetry. This follows OTEL best practices where:
    - Libraries/SDKs get tracers from the global provider
    - Applications configure the TracerProvider

    If no TracerProvider is configured, the ProxyTracerProvider (default) will
    return a NoOp tracer, effectively disabling tracing. Once the application
    sets up a real TracerProvider, subsequent spans will be recorded.

    Returns:
        Tuple[bool, Tracer]: (tracing_enabled, tracer)
            - tracing_enabled is True if a real TracerProvider is configured
            - tracer is always valid (may be NoOp if no provider configured)
    """
    tracer_provider = trace.get_tracer_provider()
    tracer = tracer_provider.get_tracer(MISTRAL_SDK_OTEL_TRACER_NAME)

    # Tracing is considered enabled if we have a real TracerProvider (not the default proxy)
    tracing_enabled = not isinstance(tracer_provider, trace.ProxyTracerProvider)

    return tracing_enabled, tracer


def get_traced_request_and_span(
    tracing_enabled: bool,
    tracer: Tracer,
    span: Span | None,
    operation_id: str,
    request: httpx.Request,
) -> tuple[httpx.Request, Span | None]:
    if not tracing_enabled:
        return request, span

    try:
        span = tracer.start_span(name=operation_id)
        span.set_attributes({"agent.trace.public": ""})
        # Propagate gen_ai.conversation.id from OTEL baggage if present
        conversation_id = get_baggage(gen_ai_attributes.GEN_AI_CONVERSATION_ID)
        if conversation_id:
            span.set_attribute(
                gen_ai_attributes.GEN_AI_CONVERSATION_ID, str(conversation_id)
            )
        # Inject the span context into the request headers to be used by the backend service to continue the trace
        propagate.inject(request.headers, context=set_span_in_context(span))
        span = enrich_span_from_request(span, operation_id, request)
    except Exception:
        logger.warning(
            "%s %s",
            TracingErrors.FAILED_TO_CREATE_SPAN_FOR_REQUEST,
            traceback.format_exc() if MISTRAL_SDK_DEBUG_TRACING else DEBUG_HINT,
        )
        if span:
            end_span(span=span)
        span = None

    return request, span


def get_traced_response(
    tracing_enabled: bool,
    tracer: Tracer,
    span: Span | None,
    operation_id: str,
    response: httpx.Response,
) -> httpx.Response:
    if not tracing_enabled or not span:
        return response
    try:
        span.set_status(Status(StatusCode.OK))
        span.set_attribute(
            http_attributes.HTTP_RESPONSE_STATUS_CODE, response.status_code
        )
        is_stream_response = not response.is_closed and not response.is_stream_consumed
        if is_stream_response:
            return TracedResponse.from_response(
                resp=response, span=span, tracer=tracer, operation_id=operation_id
            )
        if response.content:
            response_data = json.loads(response.content)
            _enrich_span_from_response(tracer, span, operation_id, response_data)
    except Exception:
        logger.warning(
            "%s %s",
            TracingErrors.FAILED_TO_ENRICH_SPAN_WITH_RESPONSE,
            traceback.format_exc() if MISTRAL_SDK_DEBUG_TRACING else DEBUG_HINT,
        )
    if span:
        end_span(span=span)
    return response


def get_response_and_error(
    tracing_enabled: bool,
    tracer: Tracer,
    span: Span | None,
    operation_id: str,
    response: httpx.Response,
    error: Exception | None,
) -> tuple[httpx.Response, Exception | None]:
    if not tracing_enabled or not span:
        return response, error
    try:
        if error:
            span.record_exception(error)
            span.set_status(Status(StatusCode.ERROR, str(error)))
        if response.content:
            response_body = json.loads(response.content)
            if response_body.get("object", "") == "error":
                if error_msg := response_body.get("message", ""):
                    error_type = response_body.get("type", "")
                    span.set_status(Status(StatusCode.ERROR, error_msg))
                    span.add_event(
                        "exception",
                        {
                            "exception.type": error_type or "api_error",
                            "exception.message": error_msg,
                        },
                    )
                    attributes = {
                        http_attributes.HTTP_RESPONSE_STATUS_CODE: response.status_code,
                        error_attributes.ERROR_TYPE: error_type,
                        MistralAIAttributes.MISTRAL_AI_ERROR_CODE: response_body.get(
                            "code", ""
                        ),
                    }
                    for attribute, value in attributes.items():
                        if value:
                            span.set_attribute(attribute, value)
        span.end()
        span = None
    except Exception:
        logger.warning(
            "%s %s",
            TracingErrors.FAILED_TO_HANDLE_ERROR_IN_SPAN,
            traceback.format_exc() if MISTRAL_SDK_DEBUG_TRACING else DEBUG_HINT,
        )

        if span:
            span.end()
            span = None
    return response, error


def end_span(span: Span) -> None:
    try:
        span.end()
    except Exception:
        logger.warning(
            "%s %s",
            TracingErrors.FAILED_TO_END_SPAN,
            traceback.format_exc() if MISTRAL_SDK_DEBUG_TRACING else DEBUG_HINT,
        )


class TracedResponse(httpx.Response):
    """Subclass of httpx.Response that accumulates streamed SSE bytes and
    enriches the OTEL span with response attributes when the stream is closed.
    """

    span: Span | None
    tracer: Tracer
    operation_id: str
    _accumulated_sse: bytearray

    def __init__(
        self,
        *args,
        span: Span | None,
        tracer: Tracer,
        operation_id: str = "",
        **kwargs,
    ) -> None:
        super().__init__(*args, **kwargs)
        self.span = span
        self.tracer = tracer
        self.operation_id = operation_id
        self._accumulated_sse = bytearray()

    def iter_bytes(self, *args, **kwargs):
        for chunk in super().iter_bytes(*args, **kwargs):
            self._accumulated_sse.extend(chunk)
            yield chunk

    async def aiter_bytes(self, *args, **kwargs):
        async for chunk in super().aiter_bytes(*args, **kwargs):
            self._accumulated_sse.extend(chunk)
            yield chunk

    def close(self) -> None:
        self._finalize_span()
        super().close()

    async def aclose(self) -> None:
        self._finalize_span()
        await super().aclose()

    def _finalize_span(self) -> None:
        """Enrich and end the span after the stream has been fully consumed."""
        if not self.span:
            return
        try:
            chunks = parse_sse_chunks(bytes(self._accumulated_sse))
            if chunks:
                response_data = accumulate_chunks_to_response_dict(chunks)
                _enrich_span_from_response(
                    self.tracer, self.span, self.operation_id, response_data
                )
        except Exception:
            logger.warning(
                "%s %s",
                TracingErrors.FAILED_TO_ENRICH_SPAN_WITH_RESPONSE,
                traceback.format_exc() if MISTRAL_SDK_DEBUG_TRACING else DEBUG_HINT,
            )
        end_span(span=self.span)
        self.span = None

    @classmethod
    def from_response(
        cls,
        resp: httpx.Response,
        span: Span | None,
        tracer: Tracer,
        operation_id: str = "",
    ) -> "TracedResponse":
        # Bypass __init__ to steal the live httpx stream/connection via __dict__ copy.
        # Keep tracing field assignments in sync with __init__.
        traced_resp = cls.__new__(cls)
        traced_resp.__dict__ = copy.copy(resp.__dict__)
        traced_resp.span = span
        traced_resp.tracer = tracer
        traced_resp.operation_id = operation_id
        traced_resp._accumulated_sse = bytearray()

        return traced_resp



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .assistantmessage import AssistantMessage, AssistantMessageTypedDict
from mistralai.gcp.client.types import BaseModel, UnrecognizedStr
from typing import Literal, Union
from typing_extensions import TypedDict


ChatCompletionChoiceFinishReason = Union[
    Literal[
        "stop",
        "length",
        "model_length",
        "error",
        "tool_calls",
    ],
    UnrecognizedStr,
]


class ChatCompletionChoiceTypedDict(TypedDict):
    index: int
    message: AssistantMessageTypedDict
    finish_reason: ChatCompletionChoiceFinishReason


class ChatCompletionChoice(BaseModel):
    index: int

    message: AssistantMessage

    finish_reason: ChatCompletionChoiceFinishReason



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 3df3767a7b93

from __future__ import annotations
from typing import Any, Dict, Union
from typing_extensions import TypeAliasType


FunctionCallEntryArgumentsTypedDict = TypeAliasType(
    "FunctionCallEntryArgumentsTypedDict", Union[Dict[str, Any], str]
)


FunctionCallEntryArguments = TypeAliasType(
    "FunctionCallEntryArguments", Union[Dict[str, Any], str]
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 81919086e371

from __future__ import annotations
from .filepurpose import FilePurpose
from .filevisibility import FileVisibility
from .sampletype import SampleType
from .source import Source
from mistralai.client.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
import pydantic
from pydantic import model_serializer
from typing_extensions import Annotated, NotRequired, TypedDict


class GetFileResponseTypedDict(TypedDict):
    id: str
    r"""The unique identifier of the file."""
    object: str
    r"""The object type, which is always \"file\"."""
    size_bytes: int
    r"""The size of the file, in bytes."""
    created_at: int
    r"""The UNIX timestamp (in seconds) of the event."""
    filename: str
    r"""The name of the uploaded file."""
    purpose: FilePurpose
    sample_type: SampleType
    source: Source
    deleted: bool
    num_lines: NotRequired[Nullable[int]]
    mimetype: NotRequired[Nullable[str]]
    signature: NotRequired[Nullable[str]]
    expires_at: NotRequired[Nullable[int]]
    visibility: NotRequired[Nullable[FileVisibility]]


class GetFileResponse(BaseModel):
    id: str
    r"""The unique identifier of the file."""

    object: str
    r"""The object type, which is always \"file\"."""

    size_bytes: Annotated[int, pydantic.Field(alias="bytes")]
    r"""The size of the file, in bytes."""

    created_at: int
    r"""The UNIX timestamp (in seconds) of the event."""

    filename: str
    r"""The name of the uploaded file."""

    purpose: FilePurpose

    sample_type: SampleType

    source: Source

    deleted: bool

    num_lines: OptionalNullable[int] = UNSET

    mimetype: OptionalNullable[str] = UNSET

    signature: OptionalNullable[str] = UNSET

    expires_at: OptionalNullable[int] = UNSET

    visibility: OptionalNullable[FileVisibility] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["num_lines", "mimetype", "signature", "expires_at", "visibility"]
        )
        nullable_fields = set(
            ["num_lines", "mimetype", "signature", "expires_at", "visibility"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    GetFileResponse.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 0330b8930f65

from __future__ import annotations
from mistralai.client.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from mistralai.client.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class LegacyJobMetadataTypedDict(TypedDict):
    details: str
    expected_duration_seconds: NotRequired[Nullable[int]]
    r"""The approximated time (in seconds) for the fine-tuning process to complete."""
    cost: NotRequired[Nullable[float]]
    r"""The cost of the fine-tuning job."""
    cost_currency: NotRequired[Nullable[str]]
    r"""The currency used for the fine-tuning job cost."""
    train_tokens_per_step: NotRequired[Nullable[int]]
    r"""The number of tokens consumed by one training step."""
    train_tokens: NotRequired[Nullable[int]]
    r"""The total number of tokens used during the fine-tuning process."""
    data_tokens: NotRequired[Nullable[int]]
    r"""The total number of tokens in the training dataset."""
    estimated_start_time: NotRequired[Nullable[int]]
    deprecated: NotRequired[bool]
    epochs: NotRequired[Nullable[float]]
    r"""The number of complete passes through the entire training dataset."""
    training_steps: NotRequired[Nullable[int]]
    r"""The number of training steps to perform. A training step refers to a single update of the model weights during the fine-tuning process. This update is typically calculated using a batch of samples from the training dataset."""
    object: Literal["job.metadata"]


class LegacyJobMetadata(BaseModel):
    details: str

    expected_duration_seconds: OptionalNullable[int] = UNSET
    r"""The approximated time (in seconds) for the fine-tuning process to complete."""

    cost: OptionalNullable[float] = UNSET
    r"""The cost of the fine-tuning job."""

    cost_currency: OptionalNullable[str] = UNSET
    r"""The currency used for the fine-tuning job cost."""

    train_tokens_per_step: OptionalNullable[int] = UNSET
    r"""The number of tokens consumed by one training step."""

    train_tokens: OptionalNullable[int] = UNSET
    r"""The total number of tokens used during the fine-tuning process."""

    data_tokens: OptionalNullable[int] = UNSET
    r"""The total number of tokens in the training dataset."""

    estimated_start_time: OptionalNullable[int] = UNSET

    deprecated: Optional[bool] = True

    epochs: OptionalNullable[float] = UNSET
    r"""The number of complete passes through the entire training dataset."""

    training_steps: OptionalNullable[int] = UNSET
    r"""The number of training steps to perform. A training step refers to a single update of the model weights during the fine-tuning process. This update is typically calculated using a batch of samples from the training dataset."""

    object: Annotated[
        Annotated[
            Optional[Literal["job.metadata"]],
            AfterValidator(validate_const("job.metadata")),
        ],
        pydantic.Field(alias="object"),
    ] = "job.metadata"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "expected_duration_seconds",
                "cost",
                "cost_currency",
                "train_tokens_per_step",
                "train_tokens",
                "data_tokens",
                "estimated_start_time",
                "deprecated",
                "epochs",
                "training_steps",
                "object",
            ]
        )
        nullable_fields = set(
            [
                "expected_duration_seconds",
                "cost",
                "cost_currency",
                "train_tokens_per_step",
                "train_tokens",
                "data_tokens",
                "estimated_start_time",
                "epochs",
                "training_steps",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    LegacyJobMetadata.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: a5f0bad3ba10

from __future__ import annotations
from mistralai.client.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class SearchChatCompletionEventIdsResponseTypedDict(TypedDict):
    completion_event_ids: List[str]


class SearchChatCompletionEventIdsResponse(BaseModel):
    completion_event_ids: List[str]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: e730005e44cb

from __future__ import annotations
from mistralai.client.types import BaseModel
from typing_extensions import TypedDict


class DocumentTextContentTypedDict(TypedDict):
    text: str


class DocumentTextContent(BaseModel):
    text: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: d396e018c804

from __future__ import annotations
from .importdatasetfromdatasetrequest import (
    ImportDatasetFromDatasetRequest,
    ImportDatasetFromDatasetRequestTypedDict,
)
from mistralai.client.types import BaseModel
from mistralai.client.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class PostDatasetRecordsFromDatasetV1ObservabilityDatasetsDatasetIDImportsFromDatasetPostRequestTypedDict(
    TypedDict
):
    dataset_id: str
    import_dataset_from_dataset_request: ImportDatasetFromDatasetRequestTypedDict


class PostDatasetRecordsFromDatasetV1ObservabilityDatasetsDatasetIDImportsFromDatasetPostRequest(
    BaseModel
):
    dataset_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    import_dataset_from_dataset_request: Annotated[
        ImportDatasetFromDatasetRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 6ea6a0dab32f

from __future__ import annotations
from mistralai.client.types import BaseModel
from mistralai.client.utils import FieldMetadata, PathParamMetadata
from typing_extensions import Annotated, TypedDict


class GetDatasetRecordV1ObservabilityDatasetRecordsDatasetRecordIDGetRequestTypedDict(
    TypedDict
):
    dataset_record_id: str


class GetDatasetRecordV1ObservabilityDatasetRecordsDatasetRecordIDGetRequest(BaseModel):
    dataset_record_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: bb511e7c628b

from __future__ import annotations
from .moderationllmaction import ModerationLLMAction
from .moderationllmv2categorythresholds import (
    ModerationLlmv2CategoryThresholds,
    ModerationLlmv2CategoryThresholdsTypedDict,
)
from mistralai.client.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ModerationLlmv2ConfigTypedDict(TypedDict):
    model_name: NotRequired[str]
    r"""Override model name. Should be omitted in general."""
    custom_category_thresholds: NotRequired[
        Nullable[ModerationLlmv2CategoryThresholdsTypedDict]
    ]
    ignore_other_categories: NotRequired[bool]
    r"""If true, only evaluate categories in custom_category_thresholds; others are ignored."""
    action: NotRequired[ModerationLLMAction]


class ModerationLlmv2Config(BaseModel):
    model_name: Optional[str] = "mistral-moderation-2603"
    r"""Override model name. Should be omitted in general."""

    custom_category_thresholds: OptionalNullable[ModerationLlmv2CategoryThresholds] = (
        UNSET
    )

    ignore_other_categories: Optional[bool] = False
    r"""If true, only evaluate categories in custom_category_thresholds; others are ignored."""

    action: Optional[ModerationLLMAction] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "model_name",
                "custom_category_thresholds",
                "ignore_other_categories",
                "action",
            ]
        )
        nullable_fields = set(["custom_category_thresholds"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from mistralai.gcp.client import errors, models, utils
from mistralai.gcp.client._hooks import HookContext
from mistralai.gcp.client.types import OptionalNullable, UNSET
from mistralai.gcp.client.utils import eventstreaming
from mistralai.gcp.client.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, List, Mapping, Optional, Union


class Chat(BaseSDK):
    r"""Chat Completion API."""

    def stream(
        self,
        *,
        model: str,
        messages: Union[
            List[models.ChatCompletionStreamRequestMessage],
            List[models.ChatCompletionStreamRequestMessageTypedDict],
        ],
        temperature: OptionalNullable[float] = UNSET,
        top_p: Optional[float] = None,
        max_tokens: OptionalNullable[int] = UNSET,
        stream: Optional[bool] = True,
        stop: Optional[
            Union[
                models.ChatCompletionStreamRequestStop,
                models.ChatCompletionStreamRequestStopTypedDict,
            ]
        ] = None,
        random_seed: OptionalNullable[int] = UNSET,
        metadata: OptionalNullable[Dict[str, Any]] = UNSET,
        response_format: Optional[
            Union[models.ResponseFormat, models.ResponseFormatTypedDict]
        ] = None,
        tools: OptionalNullable[
            Union[List[models.Tool], List[models.ToolTypedDict]]
        ] = UNSET,
        tool_choice: Optional[
            Union[
                models.ChatCompletionStreamRequestToolChoice,
                models.ChatCompletionStreamRequestToolChoiceTypedDict,
            ]
        ] = None,
        presence_penalty: Optional[float] = None,
        frequency_penalty: Optional[float] = None,
        n: OptionalNullable[int] = UNSET,
        prediction: Optional[
            Union[models.Prediction, models.PredictionTypedDict]
        ] = None,
        parallel_tool_calls: Optional[bool] = None,
        prompt_mode: OptionalNullable[models.MistralPromptMode] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> eventstreaming.EventStream[models.CompletionEvent]:
        r"""Stream chat completion

        Mistral AI provides the ability to stream responses back to a client in order to allow partial results for certain requests. Tokens will be sent as data-only server-sent events as they become available, with the stream terminated by a data: [DONE] message. Otherwise, the server will hold the request open until the timeout or until completion, with the response containing the full result as JSON.

        :param model: ID of the model to use. You can use the [List Available Models](/api/#tag/models/operation/list_models_v1_models_get) API to see all of your available models, or see our [Model overview](/models) for model descriptions.
        :param messages: The prompt(s) to generate completions for, encoded as a list of dict with role and content.
        :param temperature: What sampling temperature to use, we recommend between 0.0 and 0.7. Higher values like 0.7 will make the output more random, while lower values like 0.2 will make it more focused and deterministic. We generally recommend altering this or `top_p` but not both. The default value varies depending on the model you are targeting. Call the `/models` endpoint to retrieve the appropriate value.
        :param top_p: Nucleus sampling, where the model considers the results of the tokens with `top_p` probability mass. So 0.1 means only the tokens comprising the top 10% probability mass are considered. We generally recommend altering this or `temperature` but not both.
        :param max_tokens: The maximum number of tokens to generate in the completion. The token count of your prompt plus `max_tokens` cannot exceed the model's context length.
        :param stream:
        :param stop: Stop generation if this token is detected. Or if one of these tokens is detected when providing an array
        :param random_seed: The seed to use for random sampling. If set, different calls will generate deterministic results.
        :param metadata:
        :param response_format: Specify the format that the model must output. By default it will use `{ \"type\": \"text\" }`. Setting to `{ \"type\": \"json_object\" }` enables JSON mode, which guarantees the message the model generates is in JSON. When using JSON mode you MUST also instruct the model to produce JSON yourself with a system or a user message. Setting to `{ \"type\": \"json_schema\" }` enables JSON schema mode, which guarantees the message the model generates is in JSON and follows the schema you provide.
        :param tools: A list of tools the model may call. Use this to provide a list of functions the model may generate JSON inputs for.
        :param tool_choice: Controls which (if any) tool is called by the model. `none` means the model will not call any tool and instead generates a message. `auto` means the model can pick between generating a message or calling one or more tools. `any` or `required` means the model must call one or more tools. Specifying a particular tool via `{\"type\": \"function\", \"function\": {\"name\": \"my_function\"}}` forces the model to call that tool.
        :param presence_penalty: The `presence_penalty` determines how much the model penalizes the repetition of words or phrases. A higher presence penalty encourages the model to use a wider variety of words and phrases, making the output more diverse and creative.
        :param frequency_penalty: The `frequency_penalty` penalizes the repetition of words based on their frequency in the generated text. A higher frequency penalty discourages the model from repeating words that have already appeared frequently in the output, promoting diversity and reducing repetition.
        :param n: Number of completions to return for each request, input tokens are only billed once.
        :param prediction: Enable users to specify an expected completion, optimizing response times by leveraging known or predictable content.
        :param parallel_tool_calls: Whether to enable parallel function calling during tool use, when enabled the model can call multiple tools in parallel.
        :param prompt_mode: Allows toggling between the reasoning mode and no system prompt. When set to `reasoning` the system prompt for reasoning models will be used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ChatCompletionStreamRequest(
            model=model,
            temperature=temperature,
            top_p=top_p,
            max_tokens=max_tokens,
            stream=stream,
            stop=stop,
            random_seed=random_seed,
            metadata=metadata,
            messages=utils.get_pydantic_model(
                messages, List[models.ChatCompletionStreamRequestMessage]
            ),
            response_format=utils.get_pydantic_model(
                response_format, Optional[models.ResponseFormat]
            ),
            tools=utils.get_pydantic_model(tools, OptionalNullable[List[models.Tool]]),
            tool_choice=utils.get_pydantic_model(
                tool_choice, Optional[models.ChatCompletionStreamRequestToolChoice]
            ),
            presence_penalty=presence_penalty,
            frequency_penalty=frequency_penalty,
            n=n,
            prediction=utils.get_pydantic_model(
                prediction, Optional[models.Prediction]
            ),
            parallel_tool_calls=parallel_tool_calls,
            prompt_mode=prompt_mode,
        )

        req = self._build_request(
            method="POST",
            path="/streamRawPredict",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="text/event-stream",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.ChatCompletionStreamRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="stream_chat",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            stream=True,
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "text/event-stream"):
            return eventstreaming.EventStream(
                http_res,
                lambda raw: utils.unmarshal_json(raw, models.CompletionEvent),
                sentinel="[DONE]",
                client_ref=self,
            )
        if utils.match_response(http_res, "422", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res, http_res_text
            )
            raise errors.HTTPValidationError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        http_res_text = utils.stream_to_text(http_res)
        raise errors.SDKError("Unexpected response received", http_res, http_res_text)

    async def stream_async(
        self,
        *,
        model: str,
        messages: Union[
            List[models.ChatCompletionStreamRequestMessage],
            List[models.ChatCompletionStreamRequestMessageTypedDict],
        ],
        temperature: OptionalNullable[float] = UNSET,
        top_p: Optional[float] = None,
        max_tokens: OptionalNullable[int] = UNSET,
        stream: Optional[bool] = True,
        stop: Optional[
            Union[
                models.ChatCompletionStreamRequestStop,
                models.ChatCompletionStreamRequestStopTypedDict,
            ]
        ] = None,
        random_seed: OptionalNullable[int] = UNSET,
        metadata: OptionalNullable[Dict[str, Any]] = UNSET,
        response_format: Optional[
            Union[models.ResponseFormat, models.ResponseFormatTypedDict]
        ] = None,
        tools: OptionalNullable[
            Union[List[models.Tool], List[models.ToolTypedDict]]
        ] = UNSET,
        tool_choice: Optional[
            Union[
                models.ChatCompletionStreamRequestToolChoice,
                models.ChatCompletionStreamRequestToolChoiceTypedDict,
            ]
        ] = None,
        presence_penalty: Optional[float] = None,
        frequency_penalty: Optional[float] = None,
        n: OptionalNullable[int] = UNSET,
        prediction: Optional[
            Union[models.Prediction, models.PredictionTypedDict]
        ] = None,
        parallel_tool_calls: Optional[bool] = None,
        prompt_mode: OptionalNullable[models.MistralPromptMode] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> eventstreaming.EventStreamAsync[models.CompletionEvent]:
        r"""Stream chat completion

        Mistral AI provides the ability to stream responses back to a client in order to allow partial results for certain requests. Tokens will be sent as data-only server-sent events as they become available, with the stream terminated by a data: [DONE] message. Otherwise, the server will hold the request open until the timeout or until completion, with the response containing the full result as JSON.

        :param model: ID of the model to use. You can use the [List Available Models](/api/#tag/models/operation/list_models_v1_models_get) API to see all of your available models, or see our [Model overview](/models) for model descriptions.
        :param messages: The prompt(s) to generate completions for, encoded as a list of dict with role and content.
        :param temperature: What sampling temperature to use, we recommend between 0.0 and 0.7. Higher values like 0.7 will make the output more random, while lower values like 0.2 will make it more focused and deterministic. We generally recommend altering this or `top_p` but not both. The default value varies depending on the model you are targeting. Call the `/models` endpoint to retrieve the appropriate value.
        :param top_p: Nucleus sampling, where the model considers the results of the tokens with `top_p` probability mass. So 0.1 means only the tokens comprising the top 10% probability mass are considered. We generally recommend altering this or `temperature` but not both.
        :param max_tokens: The maximum number of tokens to generate in the completion. The token count of your prompt plus `max_tokens` cannot exceed the model's context length.
        :param stream:
        :param stop: Stop generation if this token is detected. Or if one of these tokens is detected when providing an array
        :param random_seed: The seed to use for random sampling. If set, different calls will generate deterministic results.
        :param metadata:
        :param response_format: Specify the format that the model must output. By default it will use `{ \"type\": \"text\" }`. Setting to `{ \"type\": \"json_object\" }` enables JSON mode, which guarantees the message the model generates is in JSON. When using JSON mode you MUST also instruct the model to produce JSON yourself with a system or a user message. Setting to `{ \"type\": \"json_schema\" }` enables JSON schema mode, which guarantees the message the model generates is in JSON and follows the schema you provide.
        :param tools: A list of tools the model may call. Use this to provide a list of functions the model may generate JSON inputs for.
        :param tool_choice: Controls which (if any) tool is called by the model. `none` means the model will not call any tool and instead generates a message. `auto` means the model can pick between generating a message or calling one or more tools. `any` or `required` means the model must call one or more tools. Specifying a particular tool via `{\"type\": \"function\", \"function\": {\"name\": \"my_function\"}}` forces the model to call that tool.
        :param presence_penalty: The `presence_penalty` determines how much the model penalizes the repetition of words or phrases. A higher presence penalty encourages the model to use a wider variety of words and phrases, making the output more diverse and creative.
        :param frequency_penalty: The `frequency_penalty` penalizes the repetition of words based on their frequency in the generated text. A higher frequency penalty discourages the model from repeating words that have already appeared frequently in the output, promoting diversity and reducing repetition.
        :param n: Number of completions to return for each request, input tokens are only billed once.
        :param prediction: Enable users to specify an expected completion, optimizing response times by leveraging known or predictable content.
        :param parallel_tool_calls: Whether to enable parallel function calling during tool use, when enabled the model can call multiple tools in parallel.
        :param prompt_mode: Allows toggling between the reasoning mode and no system prompt. When set to `reasoning` the system prompt for reasoning models will be used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ChatCompletionStreamRequest(
            model=model,
            temperature=temperature,
            top_p=top_p,
            max_tokens=max_tokens,
            stream=stream,
            stop=stop,
            random_seed=random_seed,
            metadata=metadata,
            messages=utils.get_pydantic_model(
                messages, List[models.ChatCompletionStreamRequestMessage]
            ),
            response_format=utils.get_pydantic_model(
                response_format, Optional[models.ResponseFormat]
            ),
            tools=utils.get_pydantic_model(tools, OptionalNullable[List[models.Tool]]),
            tool_choice=utils.get_pydantic_model(
                tool_choice, Optional[models.ChatCompletionStreamRequestToolChoice]
            ),
            presence_penalty=presence_penalty,
            frequency_penalty=frequency_penalty,
            n=n,
            prediction=utils.get_pydantic_model(
                prediction, Optional[models.Prediction]
            ),
            parallel_tool_calls=parallel_tool_calls,
            prompt_mode=prompt_mode,
        )

        req = self._build_request_async(
            method="POST",
            path="/streamRawPredict",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="text/event-stream",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.ChatCompletionStreamRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="stream_chat",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            stream=True,
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "text/event-stream"):
            return eventstreaming.EventStreamAsync(
                http_res,
                lambda raw: utils.unmarshal_json(raw, models.CompletionEvent),
                sentinel="[DONE]",
                client_ref=self,
            )
        if utils.match_response(http_res, "422", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res, http_res_text
            )
            raise errors.HTTPValidationError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        http_res_text = await utils.stream_to_text_async(http_res)
        raise errors.SDKError("Unexpected response received", http_res, http_res_text)

    def complete(
        self,
        *,
        model: str,
        messages: Union[
            List[models.ChatCompletionRequestMessage],
            List[models.ChatCompletionRequestMessageTypedDict],
        ],
        temperature: OptionalNullable[float] = UNSET,
        top_p: Optional[float] = None,
        max_tokens: OptionalNullable[int] = UNSET,
        stream: Optional[bool] = False,
        stop: Optional[
            Union[
                models.ChatCompletionRequestStop,
                models.ChatCompletionRequestStopTypedDict,
            ]
        ] = None,
        random_seed: OptionalNullable[int] = UNSET,
        metadata: OptionalNullable[Dict[str, Any]] = UNSET,
        response_format: Optional[
            Union[models.ResponseFormat, models.ResponseFormatTypedDict]
        ] = None,
        tools: OptionalNullable[
            Union[List[models.Tool], List[models.ToolTypedDict]]
        ] = UNSET,
        tool_choice: Optional[
            Union[
                models.ChatCompletionRequestToolChoice,
                models.ChatCompletionRequestToolChoiceTypedDict,
            ]
        ] = None,
        presence_penalty: Optional[float] = None,
        frequency_penalty: Optional[float] = None,
        n: OptionalNullable[int] = UNSET,
        prediction: Optional[
            Union[models.Prediction, models.PredictionTypedDict]
        ] = None,
        parallel_tool_calls: Optional[bool] = None,
        prompt_mode: OptionalNullable[models.MistralPromptMode] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ChatCompletionResponse:
        r"""Chat Completion

        :param model: ID of the model to use. You can use the [List Available Models](/api/#tag/models/operation/list_models_v1_models_get) API to see all of your available models, or see our [Model overview](/models) for model descriptions.
        :param messages: The prompt(s) to generate completions for, encoded as a list of dict with role and content.
        :param temperature: What sampling temperature to use, we recommend between 0.0 and 0.7. Higher values like 0.7 will make the output more random, while lower values like 0.2 will make it more focused and deterministic. We generally recommend altering this or `top_p` but not both. The default value varies depending on the model you are targeting. Call the `/models` endpoint to retrieve the appropriate value.
        :param top_p: Nucleus sampling, where the model considers the results of the tokens with `top_p` probability mass. So 0.1 means only the tokens comprising the top 10% probability mass are considered. We generally recommend altering this or `temperature` but not both.
        :param max_tokens: The maximum number of tokens to generate in the completion. The token count of your prompt plus `max_tokens` cannot exceed the model's context length.
        :param stream: Whether to stream back partial progress. If set, tokens will be sent as data-only server-side events as they become available, with the stream terminated by a data: [DONE] message. Otherwise, the server will hold the request open until the timeout or until completion, with the response containing the full result as JSON.
        :param stop: Stop generation if this token is detected. Or if one of these tokens is detected when providing an array
        :param random_seed: The seed to use for random sampling. If set, different calls will generate deterministic results.
        :param metadata:
        :param response_format: Specify the format that the model must output. By default it will use `{ \"type\": \"text\" }`. Setting to `{ \"type\": \"json_object\" }` enables JSON mode, which guarantees the message the model generates is in JSON. When using JSON mode you MUST also instruct the model to produce JSON yourself with a system or a user message. Setting to `{ \"type\": \"json_schema\" }` enables JSON schema mode, which guarantees the message the model generates is in JSON and follows the schema you provide.
        :param tools: A list of tools the model may call. Use this to provide a list of functions the model may generate JSON inputs for.
        :param tool_choice: Controls which (if any) tool is called by the model. `none` means the model will not call any tool and instead generates a message. `auto` means the model can pick between generating a message or calling one or more tools. `any` or `required` means the model must call one or more tools. Specifying a particular tool via `{\"type\": \"function\", \"function\": {\"name\": \"my_function\"}}` forces the model to call that tool.
        :param presence_penalty: The `presence_penalty` determines how much the model penalizes the repetition of words or phrases. A higher presence penalty encourages the model to use a wider variety of words and phrases, making the output more diverse and creative.
        :param frequency_penalty: The `frequency_penalty` penalizes the repetition of words based on their frequency in the generated text. A higher frequency penalty discourages the model from repeating words that have already appeared frequently in the output, promoting diversity and reducing repetition.
        :param n: Number of completions to return for each request, input tokens are only billed once.
        :param prediction: Enable users to specify an expected completion, optimizing response times by leveraging known or predictable content.
        :param parallel_tool_calls: Whether to enable parallel function calling during tool use, when enabled the model can call multiple tools in parallel.
        :param prompt_mode: Allows toggling between the reasoning mode and no system prompt. When set to `reasoning` the system prompt for reasoning models will be used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ChatCompletionRequest(
            model=model,
            temperature=temperature,
            top_p=top_p,
            max_tokens=max_tokens,
            stream=stream,
            stop=stop,
            random_seed=random_seed,
            metadata=metadata,
            messages=utils.get_pydantic_model(
                messages, List[models.ChatCompletionRequestMessage]
            ),
            response_format=utils.get_pydantic_model(
                response_format, Optional[models.ResponseFormat]
            ),
            tools=utils.get_pydantic_model(tools, OptionalNullable[List[models.Tool]]),
            tool_choice=utils.get_pydantic_model(
                tool_choice, Optional[models.ChatCompletionRequestToolChoice]
            ),
            presence_penalty=presence_penalty,
            frequency_penalty=frequency_penalty,
            n=n,
            prediction=utils.get_pydantic_model(
                prediction, Optional[models.Prediction]
            ),
            parallel_tool_calls=parallel_tool_calls,
            prompt_mode=prompt_mode,
        )

        req = self._build_request(
            method="POST",
            path="/rawPredict",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.ChatCompletionRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="chat_completion_v1_chat_completions_post",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.ChatCompletionResponse, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    async def complete_async(
        self,
        *,
        model: str,
        messages: Union[
            List[models.ChatCompletionRequestMessage],
            List[models.ChatCompletionRequestMessageTypedDict],
        ],
        temperature: OptionalNullable[float] = UNSET,
        top_p: Optional[float] = None,
        max_tokens: OptionalNullable[int] = UNSET,
        stream: Optional[bool] = False,
        stop: Optional[
            Union[
                models.ChatCompletionRequestStop,
                models.ChatCompletionRequestStopTypedDict,
            ]
        ] = None,
        random_seed: OptionalNullable[int] = UNSET,
        metadata: OptionalNullable[Dict[str, Any]] = UNSET,
        response_format: Optional[
            Union[models.ResponseFormat, models.ResponseFormatTypedDict]
        ] = None,
        tools: OptionalNullable[
            Union[List[models.Tool], List[models.ToolTypedDict]]
        ] = UNSET,
        tool_choice: Optional[
            Union[
                models.ChatCompletionRequestToolChoice,
                models.ChatCompletionRequestToolChoiceTypedDict,
            ]
        ] = None,
        presence_penalty: Optional[float] = None,
        frequency_penalty: Optional[float] = None,
        n: OptionalNullable[int] = UNSET,
        prediction: Optional[
            Union[models.Prediction, models.PredictionTypedDict]
        ] = None,
        parallel_tool_calls: Optional[bool] = None,
        prompt_mode: OptionalNullable[models.MistralPromptMode] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ChatCompletionResponse:
        r"""Chat Completion

        :param model: ID of the model to use. You can use the [List Available Models](/api/#tag/models/operation/list_models_v1_models_get) API to see all of your available models, or see our [Model overview](/models) for model descriptions.
        :param messages: The prompt(s) to generate completions for, encoded as a list of dict with role and content.
        :param temperature: What sampling temperature to use, we recommend between 0.0 and 0.7. Higher values like 0.7 will make the output more random, while lower values like 0.2 will make it more focused and deterministic. We generally recommend altering this or `top_p` but not both. The default value varies depending on the model you are targeting. Call the `/models` endpoint to retrieve the appropriate value.
        :param top_p: Nucleus sampling, where the model considers the results of the tokens with `top_p` probability mass. So 0.1 means only the tokens comprising the top 10% probability mass are considered. We generally recommend altering this or `temperature` but not both.
        :param max_tokens: The maximum number of tokens to generate in the completion. The token count of your prompt plus `max_tokens` cannot exceed the model's context length.
        :param stream: Whether to stream back partial progress. If set, tokens will be sent as data-only server-side events as they become available, with the stream terminated by a data: [DONE] message. Otherwise, the server will hold the request open until the timeout or until completion, with the response containing the full result as JSON.
        :param stop: Stop generation if this token is detected. Or if one of these tokens is detected when providing an array
        :param random_seed: The seed to use for random sampling. If set, different calls will generate deterministic results.
        :param metadata:
        :param response_format: Specify the format that the model must output. By default it will use `{ \"type\": \"text\" }`. Setting to `{ \"type\": \"json_object\" }` enables JSON mode, which guarantees the message the model generates is in JSON. When using JSON mode you MUST also instruct the model to produce JSON yourself with a system or a user message. Setting to `{ \"type\": \"json_schema\" }` enables JSON schema mode, which guarantees the message the model generates is in JSON and follows the schema you provide.
        :param tools: A list of tools the model may call. Use this to provide a list of functions the model may generate JSON inputs for.
        :param tool_choice: Controls which (if any) tool is called by the model. `none` means the model will not call any tool and instead generates a message. `auto` means the model can pick between generating a message or calling one or more tools. `any` or `required` means the model must call one or more tools. Specifying a particular tool via `{\"type\": \"function\", \"function\": {\"name\": \"my_function\"}}` forces the model to call that tool.
        :param presence_penalty: The `presence_penalty` determines how much the model penalizes the repetition of words or phrases. A higher presence penalty encourages the model to use a wider variety of words and phrases, making the output more diverse and creative.
        :param frequency_penalty: The `frequency_penalty` penalizes the repetition of words based on their frequency in the generated text. A higher frequency penalty discourages the model from repeating words that have already appeared frequently in the output, promoting diversity and reducing repetition.
        :param n: Number of completions to return for each request, input tokens are only billed once.
        :param prediction: Enable users to specify an expected completion, optimizing response times by leveraging known or predictable content.
        :param parallel_tool_calls: Whether to enable parallel function calling during tool use, when enabled the model can call multiple tools in parallel.
        :param prompt_mode: Allows toggling between the reasoning mode and no system prompt. When set to `reasoning` the system prompt for reasoning models will be used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ChatCompletionRequest(
            model=model,
            temperature=temperature,
            top_p=top_p,
            max_tokens=max_tokens,
            stream=stream,
            stop=stop,
            random_seed=random_seed,
            metadata=metadata,
            messages=utils.get_pydantic_model(
                messages, List[models.ChatCompletionRequestMessage]
            ),
            response_format=utils.get_pydantic_model(
                response_format, Optional[models.ResponseFormat]
            ),
            tools=utils.get_pydantic_model(tools, OptionalNullable[List[models.Tool]]),
            tool_choice=utils.get_pydantic_model(
                tool_choice, Optional[models.ChatCompletionRequestToolChoice]
            ),
            presence_penalty=presence_penalty,
            frequency_penalty=frequency_penalty,
            n=n,
            prediction=utils.get_pydantic_model(
                prediction, Optional[models.Prediction]
            ),
            parallel_tool_calls=parallel_tool_calls,
            prompt_mode=prompt_mode,
        )

        req = self._build_request_async(
            method="POST",
            path="/rawPredict",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.ChatCompletionRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="chat_completion_v1_chat_completions_post",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.ChatCompletionResponse, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 556ebdc33276

from __future__ import annotations
from .modelcapabilities import ModelCapabilities, ModelCapabilitiesTypedDict
from datetime import datetime
from mistralai.client.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from mistralai.client.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class BaseModelCardTypedDict(TypedDict):
    id: str
    capabilities: ModelCapabilitiesTypedDict
    r"""This is populated by Harmattan, but some fields have a name
    that we don't want to expose in the API.
    """
    object: NotRequired[str]
    created: NotRequired[int]
    owned_by: NotRequired[str]
    name: NotRequired[Nullable[str]]
    description: NotRequired[Nullable[str]]
    max_context_length: NotRequired[int]
    aliases: NotRequired[List[str]]
    deprecation: NotRequired[Nullable[datetime]]
    deprecation_replacement_model: NotRequired[Nullable[str]]
    default_model_temperature: NotRequired[Nullable[float]]
    type: Literal["base"]


class BaseModelCard(BaseModel):
    id: str

    capabilities: ModelCapabilities
    r"""This is populated by Harmattan, but some fields have a name
    that we don't want to expose in the API.
    """

    object: Optional[str] = "model"

    created: Optional[int] = None

    owned_by: Optional[str] = "mistralai"

    name: OptionalNullable[str] = UNSET

    description: OptionalNullable[str] = UNSET

    max_context_length: Optional[int] = 32768

    aliases: Optional[List[str]] = None

    deprecation: OptionalNullable[datetime] = UNSET

    deprecation_replacement_model: OptionalNullable[str] = UNSET

    default_model_temperature: OptionalNullable[float] = UNSET

    type: Annotated[
        Annotated[Literal["base"], AfterValidator(validate_const("base"))],
        pydantic.Field(alias="type"),
    ] = "base"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "object",
                "created",
                "owned_by",
                "name",
                "description",
                "max_context_length",
                "aliases",
                "deprecation",
                "deprecation_replacement_model",
                "default_model_temperature",
            ]
        )
        nullable_fields = set(
            [
                "name",
                "description",
                "deprecation",
                "deprecation_replacement_model",
                "default_model_temperature",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    BaseModelCard.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 77aea4882ccb

from __future__ import annotations
from mistralai.client.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ImportDatasetFromDatasetRequestTypedDict(TypedDict):
    dataset_record_ids: List[str]


class ImportDatasetFromDatasetRequest(BaseModel):
    dataset_record_ids: List[str]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 73ca3a446dcc

from __future__ import annotations
from mistralai.client.types import BaseModel, UNSET_SENTINEL
from mistralai.client.utils import FieldMetadata, PathParamMetadata, QueryParamMetadata
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ConnectorGetV1RequestTypedDict(TypedDict):
    connector_id_or_name: str
    fetch_customer_data: NotRequired[bool]
    r"""Fetch the customer data associated with the connector (e.g. customer secrets / config)."""
    fetch_connection_secrets: NotRequired[bool]
    r"""Fetch the general connection secrets associated with the connector."""


class ConnectorGetV1Request(BaseModel):
    connector_id_or_name: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    fetch_customer_data: Annotated[
        Optional[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = False
    r"""Fetch the customer data associated with the connector (e.g. customer secrets / config)."""

    fetch_connection_secrets: Annotated[
        Optional[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = False
    r"""Fetch the general connection secrets associated with the connector."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["fetch_customer_data", "fetch_connection_secrets"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 393fca552632

from __future__ import annotations
from mistralai.client.types import BaseModel
from typing import Any, Dict, Union
from typing_extensions import TypeAliasType, TypedDict


ArgumentsTypedDict = TypeAliasType("ArgumentsTypedDict", Union[Dict[str, Any], str])


Arguments = TypeAliasType("Arguments", Union[Dict[str, Any], str])


class FunctionCallTypedDict(TypedDict):
    name: str
    arguments: ArgumentsTypedDict


class FunctionCall(BaseModel):
    name: str

    arguments: Arguments



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .imageurl import ImageURL, ImageURLTypedDict
from mistralai.gcp.client.types import BaseModel
from mistralai.gcp.client.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal, Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


ImageURLUnionTypedDict = TypeAliasType(
    "ImageURLUnionTypedDict", Union[ImageURLTypedDict, str]
)


ImageURLUnion = TypeAliasType("ImageURLUnion", Union[ImageURL, str])


class ImageURLChunkTypedDict(TypedDict):
    r"""{\"type\":\"image_url\",\"image_url\":{\"url\":\"data:image/png;base64,iVBORw0"""

    image_url: ImageURLUnionTypedDict
    type: Literal["image_url"]


class ImageURLChunk(BaseModel):
    r"""{\"type\":\"image_url\",\"image_url\":{\"url\":\"data:image/png;base64,iVBORw0"""

    image_url: ImageURLUnion

    type: Annotated[
        Annotated[Literal["image_url"], AfterValidator(validate_const("image_url"))],
        pydantic.Field(alias="type"),
    ] = "image_url"


try:
    ImageURLChunk.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .jsonschema import JSONSchema, JSONSchemaTypedDict
from .responseformats import ResponseFormats
from mistralai.gcp.client.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ResponseFormatTypedDict(TypedDict):
    r"""Specify the format that the model must output. By default it will use `{ \"type\": \"text\" }`. Setting to `{ \"type\": \"json_object\" }` enables JSON mode, which guarantees the message the model generates is in JSON. When using JSON mode you MUST also instruct the model to produce JSON yourself with a system or a user message. Setting to `{ \"type\": \"json_schema\" }` enables JSON schema mode, which guarantees the message the model generates is in JSON and follows the schema you provide."""

    type: NotRequired[ResponseFormats]
    json_schema: NotRequired[Nullable[JSONSchemaTypedDict]]


class ResponseFormat(BaseModel):
    r"""Specify the format that the model must output. By default it will use `{ \"type\": \"text\" }`. Setting to `{ \"type\": \"json_object\" }` enables JSON mode, which guarantees the message the model generates is in JSON. When using JSON mode you MUST also instruct the model to produce JSON yourself with a system or a user message. Setting to `{ \"type\": \"json_schema\" }` enables JSON schema mode, which guarantees the message the model generates is in JSON and follows the schema you provide."""

    type: Optional[ResponseFormats] = None

    json_schema: OptionalNullable[JSONSchema] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["type", "json_schema"])
        nullable_fields = set(["json_schema"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from typing import Any, Optional, Type, TypeVar, overload

import httpx

from .serializers import unmarshal_json
from mistralai.azure.client import errors

T = TypeVar("T")


@overload
def unmarshal_json_response(
    typ: Type[T], http_res: httpx.Response, body: Optional[str] = None
) -> T: ...


@overload
def unmarshal_json_response(
    typ: Any, http_res: httpx.Response, body: Optional[str] = None
) -> Any: ...


def unmarshal_json_response(
    typ: Any, http_res: httpx.Response, body: Optional[str] = None
) -> Any:
    if body is None:
        body = http_res.text
    try:
        return unmarshal_json(body, typ)
    except Exception as e:
        raise errors.ResponseValidationError(
            "Response validation failed",
            http_res,
            e,
            body,
        ) from e



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: ac66815396b1

from __future__ import annotations
from mistralai.client.types import BaseModel
from mistralai.client.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class SpeechStreamAudioDeltaTypedDict(TypedDict):
    audio_data: str
    type: Literal["speech.audio.delta"]


class SpeechStreamAudioDelta(BaseModel):
    audio_data: str

    type: Annotated[
        Annotated[
            Literal["speech.audio.delta"],
            AfterValidator(validate_const("speech.audio.delta")),
        ],
        pydantic.Field(alias="type"),
    ] = "speech.audio.delta"


try:
    SpeechStreamAudioDelta.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .imageurlchunk import ImageURLChunk, ImageURLChunkTypedDict
from .referencechunk import ReferenceChunk, ReferenceChunkTypedDict
from .textchunk import TextChunk, TextChunkTypedDict
from functools import partial
from mistralai.gcp.client.types import BaseModel
from mistralai.gcp.client.utils.unions import parse_open_union
from pydantic import ConfigDict
from pydantic.functional_validators import BeforeValidator
from typing import Any, Literal, Union
from typing_extensions import Annotated, TypeAliasType


ContentChunkTypedDict = TypeAliasType(
    "ContentChunkTypedDict",
    Union[TextChunkTypedDict, ImageURLChunkTypedDict, ReferenceChunkTypedDict],
)


class UnknownContentChunk(BaseModel):
    r"""A ContentChunk variant the SDK doesn't recognize. Preserves the raw payload."""

    type: Literal["UNKNOWN"] = "UNKNOWN"
    raw: Any
    is_unknown: Literal[True] = True

    model_config = ConfigDict(frozen=True)


_CONTENT_CHUNK_VARIANTS: dict[str, Any] = {
    "image_url": ImageURLChunk,
    "text": TextChunk,
    "reference": ReferenceChunk,
}


ContentChunk = Annotated[
    Union[ImageURLChunk, TextChunk, ReferenceChunk, UnknownContentChunk],
    BeforeValidator(
        partial(
            parse_open_union,
            disc_key="type",
            variants=_CONTENT_CHUNK_VARIANTS,
            unknown_cls=UnknownContentChunk,
            union_name="ContentChunk",
        )
    ),
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .referencechunk import ReferenceChunk, ReferenceChunkTypedDict
from .textchunk import TextChunk, TextChunkTypedDict
from .toolreferencechunk import ToolReferenceChunk, ToolReferenceChunkTypedDict
from mistralai.azure.client.types import BaseModel, UNSET_SENTINEL
from mistralai.azure.client.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


ThinkingTypedDict = TypeAliasType(
    "ThinkingTypedDict",
    Union[TextChunkTypedDict, ReferenceChunkTypedDict, ToolReferenceChunkTypedDict],
)


Thinking = TypeAliasType(
    "Thinking", Union[TextChunk, ReferenceChunk, ToolReferenceChunk]
)


class ThinkChunkTypedDict(TypedDict):
    thinking: List[ThinkingTypedDict]
    type: Literal["thinking"]
    closed: NotRequired[bool]
    r"""Whether the thinking chunk is closed or not. Currently only used for prefixing."""


class ThinkChunk(BaseModel):
    thinking: List[Thinking]

    type: Annotated[
        Annotated[Literal["thinking"], AfterValidator(validate_const("thinking"))],
        pydantic.Field(alias="type"),
    ] = "thinking"

    closed: Optional[bool] = None
    r"""Whether the thinking chunk is closed or not. Currently only used for prefixing."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["closed"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    ThinkChunk.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 06bab279cb31

from __future__ import annotations
from .moderationobject import ModerationObject, ModerationObjectTypedDict
from mistralai.client.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ModerationResponseTypedDict(TypedDict):
    id: str
    model: str
    results: List[ModerationObjectTypedDict]


class ModerationResponse(BaseModel):
    id: str

    model: str

    results: List[ModerationObject]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: e41c04cf7422

from __future__ import annotations
from mistralai.client.types import UnrecognizedStr
from typing import Literal, Union


Roles = Union[
    Literal[
        "system",
        "user",
        "assistant",
        "tool",
    ],
    UnrecognizedStr,
]
